#!/bin/bash

DIRNAME=`dirname $0`
LABELDBFILE="$DIRNAME/label.db"
TMPFILE=/tmp/nodiskadd.tmp

returnresult ()
{
MESSAGE="<message>$2</message>"
cat << EOF
<commands>
    <command>
        <result>$1</result>
	$MESSAGE
    </command>
</commands>
EOF
}

normalize ()
{
	DIR=`dirname $1`
	NAME=`basename $1`
	echo "$DIR"/"$NAME"
}

trim ()
{
        echo $1
}

getDiskModel ()
{
	PARTITION=`echo $1 | sed -e 's/\/dev\///g'`
#	if [ -d /dev/disk/by-id/ ] ; then
#		DISKID=`ls -l  /dev/disk/by-id/ | grep $PARTITION$ | sed -e 's/^.*usb-//g' -e 's/-.*$//g'`
#		DISKDETAILS=`echo ${DISKID%_*} | sed -e 's/_/ /g'`
#		echo -n $DISKDETAILS
#	else
		DISK=`echo ${PARTITION} | cut -c 1-3`
		MODEL=`parted -s /dev/${DISK} print | grep Model | sed -e 's/Model: //g' -e 's/(.*//g'`
		echo -n $MODEL
#	fi
}

runBlkID ()
{
	RESULT=`blkid -s ${2} ${1} | cut -d '"' -f2`
	echo -n $RESULT
}

addLabelEntry ()
{
	#Backup existing file
    if [ -f $LABELDBFILE ] ; then
		cp $LABELDBFILE $LABELDBFILE~
	fi
	DISKID=${1}
	DISKLABEL=${2}
	echo "${DISKID}|${DISKLABEL}" >> $LABELDBFILE
}

changeLabelEntry ()
{
	#Backup existing file
    if [ -f $LABELDBFILE ] ; then
		cp $LABELDBFILE $LABELDBFILE~
	fi

    DISKID=${1}
	if [ -z $DISKID ] ; then
		return
	fi
	DISKLABEL=${2}
	if [ -z $DISKLABEL ] ; then
		return
	fi
	sed -e 's/\('"${DISKID}"'|\).*$/\1'"${DISKLABEL}"'/g' -i $LABELDBFILE
}

resetLabelRegistry ()
{
	#Backup existing file
    if [ -f $LABELDBFILE ] ; then
		cp $LABELDBFILE $LABELDBFILE~
	fi

	echo > $LABELDBFILE
}

getLabelEntry ()
{
	if [ ! -f $LABELDBFILE ] ; then
		return
	fi
	
	DISKID=$1
	if [ -z "$DISKID" ]; then
		return
	fi	
	
	LABEL=`cat $LABELDBFILE | grep $DISKID"|" | cut -d "|" -f 2`
	echo $LABEL
}


changeOtherFSLabel ()
{
	DISKUUID=`runBlkID ${1} UUID`
	LABEL=`getLabelEntry ${DISKUUID}`
	if [ -z "$LABEL" ]; then
		addLabelEntry "${DISKUUID}" "${2}"
	else
		changeLabelEntry "${DISKUUID}" "${2}"
	fi
	LABEL="${2}"
}

getDiskLabel ()
{
	FSTYPE=`runBlkID ${1} TYPE`
	PARTITIONNO=`echo "$1" | cut -c 9`
	
	if [[ "$FSTYPE" = "ext2" || "$FSTYPE" = "ext3" || "$FSTYPE" = "ext4" ]]; then
		LABEL=`e2label "$1"`
#	elif [[ "$FSTYPE" = "vfat" ]]; then
#		LABEL=`dosfslabel "$1"`
	elif [[ "$FSTYPE" = "vfat" || "$FSTYPE" = "ntfs" || "$FSTYPE" = "hfsplus" ]]; then
		DISKUUID=`runBlkID ${1} UUID`
		LABEL=`getLabelEntry ${DISKUUID}`
		LABEL=`trim "${LABEL}"`
		if [ -z "$LABEL" ]; then
			sleep 3
			LABEL=`runBlkID ${1} LABEL`
			changeOtherFSLabel "${1}" "${LABEL}"
		fi
	else
		DISKUUID=`runBlkID ${1} UUID`
		LABEL=`getLabelEntry ${DISKUUID}`
	fi
	LABEL=`trim "${LABEL}"`
	if [ -z "$LABEL" ]; then
		LABEL=`getDiskModel "${1}"`
	fi
	
	echo ${LABEL}
}

getLabelDetails ()
{
	LABEL=`getDiskLabel ${1}`
	echo "<name>${LABEL}</name>"
	echo "<editlabel>1</editlabel>"
	
}

setFSLabel ()
{
	FSTYPE=`runBlkID ${1} TYPE`
	LENGTH=`echo ${2} | wc -c`
	
	if [[ "$FSTYPE" = "ext2" || "$FSTYPE" = "ext3" || "$FSTYPE" = "ext4" ]]; then
		if [ $LENGTH -gt 17 ] ; then
			returnresult false "Ext2/Ext3/Ext4 : Label needs to be less than or equal 16 characters"		
			return
		fi
		e2label "$1" "$2"
	elif [[ "$FSTYPE" = "vfat" ]]; then
		if [ $LENGTH -gt 12 ] ; then
			returnresult false "Fat File System : Label needs to be less than or equal 11 characters"		
			return
		fi
		#dosfslabel "${1}" "${2}"
		changeOtherFSLabel "${1}" "${2}"
#	elif [[ "$FSTYPE" = "ntfs" ]]; then
#		ntfslabel "$1" "$2"
	else
		if [ $LENGTH -gt 17 ] ; then
			returnresult false "${FSTYPE} : Label needs to be less than or equal 16 characters"		
			return
		fi
		changeOtherFSLabel "${1}" "${2}"
	fi
	
	LABEL=`getDiskLabel ${1}`
	
	if [[ "$LABEL" = "$2" ]]; then
		returnresult true "Name changed successfully"	
	else
		returnresult false "Name change failed"	
	fi
	
}

getFileList()
{
	echo "<entries>"
	echo "<meta>"
	echo "<parentpath>/</parentpath>"
	echo "<isroot>1</isroot>"
    echo "<realpath>USBDRIVES</realpath>"
	echo "</meta>"
	df -h | sort | grep "dev" | grep "[sh]d" | while read part
	do
	DEVICE=`echo $part | awk  ' { print $1 } '`
	USBPATH=`echo $part | awk ' { print $6 } '`
	DIRECTORY=`dirname "$USBPATH"`
	SIZE=`echo $part | awk ' { print $2 } '`
	USED=`echo $part | awk ' { print $3 } '`
	FREE=`echo $part | awk ' { print $4 } '`
	echo "<entry>"
	
	echo "<isusb>1</isusb>"
	echo "<isroot>0</isroot>"
	echo "<ext>n/a</ext>"
	echo "<type>dir</type>"
	
	echo -n "<device>"
	echo -n $DEVICE
	echo "</device>"
	
	echo -n "<path>"
	echo -n $USBPATH
	echo "</path>"
	
	echo -n "<dirpath>"
	echo -n $DIRECTORY
	echo "</dirpath>"
	
	echo -n "<size>"
	echo -n "$FREE"B free of "$SIZE"B
	echo "</size>"
	
	echo -n "<used>"
	echo -n "$USED"B
	echo "</used>"
	
	echo -n "<free>"
	echo -n "$FREE"B
	echo "</free>"
	
	echo -n "<model>"
	getDiskModel "$DEVICE"
	echo "</model>"
	
	echo -n "<filesystem>"
	runBlkID ${DEVICE} TYPE
	echo "</filesystem>"
	
	getLabelDetails "$DEVICE"
	echo "</entry>"
	
	done
	echo "</entries>"
}

getDirList()
{
	echo "<directories>"
	df -h | sort | grep "dev" | grep "[sh]d" | while read part
	do
	DEVICE=`echo $part | awk  ' { print $1 } '`
	USBPATH=`echo $part | awk ' { print $6 } '`
	DIRECTORY=`dirname "$USBPATH"`
	SIZE=`echo $part | awk ' { print $2 } '`
	USED=`echo $part | awk ' { print $3 } '`
	FREE=`echo $part | awk ' { print $4 } '`
	echo "<directory>"
	
	echo "<isusb>1</isusb>"
		
	echo -n "<path>"
	echo -n $USBPATH
	echo "</path>"
	
	echo -n "<dirpath>"
	echo -n $USBPATH
	echo "</dirpath>"

	
	getLabelDetails "$DEVICE"
	echo "</directory>"
	
	done
	echo "</directories>"
}
###Eject a disk######
getAllDiskMountPoints ()
{
	cat /proc/mounts | sort | grep "$DISK" | while read part
	do
		VAR_A=`echo $part | awk ' { print $2 } '`
		echo -n ${VAR_A}" "
	done
}

getAllPartitions ()
{
	cat /proc/mounts | sort | grep "$DISK" | while read part
	do
		VAR_A=`echo $part | awk ' { print $1 } '`
		echo -n ${VAR_A}" "
	done
}

removeSambaShares ()
{
	BINDIR=`dirname $0`
	$BINDIR/sambaShareControl.sh removeMultiPaths ${MTPATHS} > $BINDIR/smb.conf.new
	RETCODE=$?
	testparm -s  $BINDIR/smb.conf.new >& /dev/null
	if [  "$?" != "0" ] ; then
	   	rm -f $BINDIR/smb.conf.new
 		return
	fi

	if [ $RETCODE -eq 10 ] ; then
	   mv /etc/samba/smb.conf /etc/samba/smb.conf.bak
	   mv $BINDIR/smb.conf.new /etc/samba/smb.conf 
	   /etc/init.d/samba restart >& /dev/null
	else
	   rm -f $BINDIR/smb.conf.new
	fi
}
ejectDisk()
{
	if [ -z "$1" ]; then
		returnresult false "Invalid Path"
	fi
	DEVICE=`cat /proc/mounts  | grep -m 1 "$1" | awk ' { print $1 } '`
	DISK=`echo $DEVICE | cut -c 1-8`
	
	if [ -z ${DISK} ]; then
		returnresult true "Disk ejected successfully"
		exit 0
	fi

	MTPATHS=`getAllDiskMountPoints`
	if [ "${MTPATHS}" != "" ] ; then
		removeSambaShares
	fi
	
	PARTITIONS=`getAllPartitions`
	for f in `echo ${PARTITIONS}`
	do
		umount ${f} 2>/dev/null
	done
		
	MTCOUNT=`df -h | grep "${DISK}" | wc -l`
    if [ "${MTCOUNT}" = "0" ] ; then
    	returnresult true "Disk ejected successfully"
    	exit 0
	else
    	returnresult false "Unable to eject disk"
    	exit 1
	fi
}
###End: Eject a disk######

#returns list of unmounted partitions
trim ()
{
	echo $1
}

getPartitionSize ()
{
	#echo "parted -s ${1} print | sort |sed '/^$/d' | sed '/^[A-Z].*/d' | sed '/^ .*${2} .*/!d' |  cut -c 24-30"
	for info in  `parted -s ${1} print | sort |sed '/^$/d' | sed '/^[A-Z].*/d' | tr ' ' '.'`
	do
		PARTNO=`echo $info  | cut -c 1-6 |tr -d '.'`
		if [ "$PARTNO" = "$2" ] ; then
			echo -n $info |  cut -c 24-30 |tr -d '.'
			return
		fi
	done
}
unMountedPartitions()
{
	echo "<partitions>"
	for disk in `parted -l 2>&1 | grep Disk | cut -c 11-13`
	do 
		for part in ` ls -l /dev/disk/by-uuid/ | grep $disk | cut -d ' ' -f 10 | cut -c 7-| sort`
		do
			MOUNTED=`cat /proc/mounts | cut -d ' ' -f 1 | grep "/dev/$part" `
			if [ "${MOUNTED}" = "" ] ; then
				PARTNO=`echo -n ${part#$disk}`
				MODEL=`getDiskModel "/dev/${part}"`
				SIZE=`getPartitionSize "/dev/${disk}" $PARTNO`
				echo "<partition>"
				echo -n "<no>"
				echo -n $PARTNO
				echo "</no>"
				echo -n "<device>"
				echo -n "/dev/"${part}" "
				echo "</device>"
				echo -n "<model>"
				echo -n $MODEL
				echo "</model>"
				echo -n "<size>"
				echo -n $SIZE
				echo "</size>"
				echo "<dispname>"
				echo "$MODEL - Drive $PARTNO - $SIZE"
				echo "</dispname>"
				echo "</partition>"
			fi
		done
	done
	echo "</partitions>"
}

mountPartition ()
{
	DEVICE=${1}
	if [ ! -d "$2" ]; then
		returnresult false "Access path invalid"
		exit 1
	fi

	MOUNTPATH=`normalize "$2"`
	MOUNTED=`cat /proc/mounts | cut -d ' ' -f 2 | grep "^${MOUNTPATH}$" `
	if [ "$MOUNTED" != "" ]; then
		returnresult false "Access path already in use"
		exit 1
	fi

	FILECOUNT=`ls -l ${2}| grep total | cut -f 2 -d ' '`
	if [ "${FILECOUNT}" != "0" ]; then
		returnresult false "Access path not empty"
		exit 1
	fi

	mount ${DEVICE} ${MOUNTPATH}
	RETCODE=$?
		
	if [ "$RETCODE" = "0" ]; then
		returnresult true "Drive added successfully"
		exit 0
	else
		returnresult false "Drive add failed"
		exit 1
	fi
	
}

unMountPartition ()
{
	if [ -z "$1" -o ! -d "$1" ]; then
		returnresult false "Invalid Path"
	fi
	
	DEVICE=`cat /proc/mounts  | grep "$1" | awk ' { print $1 } '`
	
	if [ -z ${DEVICE} ]; then
		returnresult true "Drive removed successfully"
		exit 0
	fi

	MTPATHS=`normalize $1`
	if [ "${MTPATHS}" != "" ] ; then
		removeSambaShares
	fi
	
	umount ${DEVICE} 2>/dev/null
		
	MTCOUNT=`df -h | grep "${DEVICE}" | wc -l`
    if [ "${MTCOUNT}" = "0" ] ; then
    	returnresult true "Drive removed successfully"
    	exit 0
	else
    	returnresult false "Drive busy ! Remove failed"
    	exit 1
	fi
	
}

formatDisk ()
{
	DISK=${1}
	if [ -z ${DISK} ]; then
                returnresult false "Disk not specified"
                exit 0
        fi
	FORMATTYPE=${2}
	if [ -z ${FORMATTYPE} ]; then
                returnresult false "Format type not specified"
                exit 0
        fi

	MOUNTED=`cat /proc/mounts | grep "${DISK}"`
        if [ "${MOUNTED}" != "" ] ; then
                returnresult false "Partition in use! Please unmount the partition to format.  "
                exit 0
        fi

	touch $TMPFILE
	# Clear all partitions
	parted -s $DISK print | awk '{if( $1+0==$1 ) print $1}'| while read part
	do
		parted -s $DISK rm $part
	done	
	
	# Create a new gpt table
	parted -s $DISK mklabel gpt

	# Create a single partition
	DISKSIZE=`parted -s $DISK print|awk '/^Disk/ {print $3}'`
	parted -s $DISK mkpart primary 0 $DISKSIZE >& /dev/null
	sleep 1
	partprobe
	rm -f $TMPFILE

	# Format the disk
	formatPartition ${DISK}1 $FORMATTYPE
}

formatPartition ()
{
	PARTITION=${1}
	if [ -z ${PARTITION} ]; then
                returnresult false "Partition not specified"
                exit 0
        fi
	FORMATTYPE=${2}
	if [ -z ${FORMATTYPE} ]; then
                returnresult false "Format type not specified"
                exit 0
        fi

	MOUNTED=`cat /proc/mounts | grep "${PARTITION}"`
	if [ "${MOUNTED}" != "" ] ; then
		returnresult false "Partition in use! Please unmount the partition to format.  "
		exit 0
	fi


	rm -f /tmp/${PARTITIONNAME}.inprogress
        rm -f /tmp/${PARTITIONNAME}.done
	
	#PARTITIONNAME=`basename ${PARTITION}`
	#if [ "$FORMATTYPE" = "fat32" ] ; then
#		FMTCMD="mkfs.vfat -F 32 ${PARTITION}"
#	elif [ "$FORMATTYPE" = "ext3" ] ; then
#		FMTCMD="mkfs.ext3 ${PARTITION}"
#	fi
#	( echo ${FORMATTYPE} > /tmp/${PARTITIONNAME}.inprogress &&
#	 ${FMTCMD} 2>/dev/null >>/tmp/${PARTITIONNAME}.inprogress ;
#	 echo 1 > /tmp/${PARTITIONNAME}.done ) &
	($DIRNAME/formatUSBDrives.sh ${PARTITION} ${FORMATTYPE} 1>/dev/null) &
	returnresult true "Formatting started."

}
formatProgress ()
{
	DISK=${1}
        if [ -z ${DISK} ]; then
                returnresult false "Disk not specified"
                exit 0
        fi

        PARTITION=${DISK}1
	PARTITIONNAME=`basename ${PARTITION}`
	INPROGRESS=0
	FMTCOMPLETE=0
	COMPLETED=0
	TOTAL=0
	if [ -f /tmp/${PARTITIONNAME}.inprogress ] && [ ! -f /tmp/${PARTITIONNAME}.done ]; then
		INPROGRESS=1
	fi
	if [ "${INPROGRESS}" = "1" ]; then
		FORMATTYPE=`head -n 1 /tmp/${PARTITIONNAME}.inprogress | awk '{print $1}'`
		if [ "$FORMATTYPE" = "fat32" ] ; then
			COMPLETED=0
			TOTAL=0
		elif [ "$FORMATTYPE" = "ext3" ] ; then
			PROGRESS=`cat /tmp/${PARTITIONNAME}.inprogress | grep "Writing inode tables" | sed -e 's/\x8/ /g' | awk '{print $NF}'`
			if [ "${PROGRESS}" != "done" ]; then
				COMPLETED=`echo ${PROGRESS} | cut -d '/' -f 1`
				TOTAL=`echo ${PROGRESS} | cut -d '/' -f 2`
			fi
		fi
	fi
	if [ -z ${INPROGRESS} ]; then
		INPROGRESS=0
	fi
	if [ -z ${COMPLETED} ]; then
		COMPLETED=0
	fi
	if [ -f /tmp/${PARTITIONNAME}.done ]; then
		# Format completed. Remove temporary files	
		rm -f /tmp/${PARTITIONNAME}.inprogress
		rm -f /tmp/${PARTITIONNAME}.done
		FMTCOMPLETE=1
		INPROGRESS=0
	fi
	echo "<format>"
	echo "	<status>"
	echo "		<type>$FORMATTYPE</type>"
	echo "		<inprogress>$INPROGRESS</inprogress>"
	echo "		<fmtcomplete>$FMTCOMPLETE</fmtcomplete>"
	echo "		<completed>$COMPLETED</completed>"
	echo "		<total>$TOTAL</total>"
	echo "	</status>"
	echo "</format>"

}

COMMAND=$1
shift

if [ "$COMMAND" = "dirlist" ] ; then
	getDirList "$@"
elif [ "$COMMAND" = "filelist" ] ; then
	getFileList "$@"
elif [ "$COMMAND" = "eject" ] ; then
	ejectDisk "$@"
elif [ "$COMMAND" = "mountpartition" ] ; then
	mountPartition "$@"	
elif [ "$COMMAND" = "unmountpartition" ] ; then
	unMountPartition "$@"	
elif [ "$COMMAND" = "unmountedpartitions" ] ; then
	unMountedPartitions "$@"
elif [ "$COMMAND" = "setfslabel" ] ; then
	setFSLabel "$@"
elif [ "$COMMAND" = "getfslabel" ] ; then
        getLabelDetails "$@"
elif [ "$COMMAND" = "format" ] ; then
	formatDisk "$@"
elif [ "$COMMAND" = "formatstatus" ] ; then
	formatProgress "$@"
elif [ "$COMMAND" = "diskmodel" ] ; then
        getDiskModel "$@"
else
	echo "Please specify the listing type {filelist|dirlist}."
fi
