#!/bin/bash

# Wrapper script for adding a share definition
# to smb.conf file

returnresult ()
{
MESSAGE="<message>$2</message>"
cat << EOF
<mediashares>
        <mediashare>
        <operation>$3</operation>
        <success>$1</success>
        $MESSAGE
        </mediashare>
</mediashares>
EOF
}

autostart ()
{
	if [ "$1" = "1" ] ; then
		update-rc.d -f ushare defaults	
	elif [ "$1" = "0" ] ; then
		update-rc.d -f ushare remove
	fi
	exit $?
}

restart ()
{
	$CTLSCRIPT stop
	pkill -9 ushare
	$CTLSCRIPT start
	exit $?
}

if [ $# -lt 1 ] ; then
        returnresult false "Missing request type" "No Operation"
        exit 2
fi

DIRNAMES=
i=0
BINDIR=`dirname $0`
CTLSCRIPT=/etc/init.d/ushare

if [ "$1" = "add" ] ; then
	if [ -z $2 ] ; then
                returnresult false "Missing directory path" "ADD"
                exit 2
        fi

	if [ -d $2 ] ; then
		CURRENTSHARES=`cat /etc/ushare.conf  |grep "^USHARE_DIR" | sed -e "s/^USHARE_DIR=//g"`
		DIRNAMES=$CURRENTSHARES","$2
		XBOX=`cat /etc/ushare.conf  |grep "^USHARE_ENABLE_XBOX" | sed -e "s/^USHARE_ENABLE_XBOX=//g"`
		DLNA=`cat /etc/ushare.conf  |grep "^USHARE_ENABLE_DLNA" | sed -e "s/^USHARE_ENABLE_DLNA=//g"`
	else
		returnresult false "Not a valid directory" "ADD"
		exit 2
	fi
elif [ "$1" = "remove" ] ; then
	if [ -z $2 ] ; then
		returnresult false "Share number not specified" "REMOVE"
		exit 2
	fi

	for share in `cat /etc/ushare.conf  |grep "^USHARE_DIR" | sed -e "s/^USHARE_DIR=//g" -e "s/,/ /g"`
	do
		if [ $i -ne $2 ] ; then		
			if [ -z $DIRNAMES ] ; then 
				DIRNAMES=$share
			else
				DIRNAMES=$DIRNAMES","$share
			fi
		fi
		i=$(( $i+1 ))
	done
	XBOX=`cat /etc/ushare.conf  |grep "^USHARE_ENABLE_XBOX" | sed -e "s/^USHARE_ENABLE_XBOX=//g"`
	DLNA=`cat /etc/ushare.conf  |grep "^USHARE_ENABLE_DLNA" | sed -e "s/^USHARE_ENABLE_DLNA=//g"`
elif [ "$1" = "xbox" ] ; then
	cat /etc/ushare.conf | sed -e "s/USHARE_ENABLE_XBOX=.*/USHARE_ENABLE_XBOX=$2/g" > /etc/ushare.conf.tmp
	mv /etc/ushare.conf.tmp /etc/ushare.conf
	restart
elif [ "$1" = "dlna" ] ; then
	cat /etc/ushare.conf | sed -e "s/USHARE_ENABLE_DLNA=.*/USHARE_ENABLE_DLNA=$2/g" > /etc/ushare.conf.tmp
	mv /etc/ushare.conf.tmp /etc/ushare.conf
	restart
elif [ "$1" = "start" ] ; then
	$CTLSCRIPT start
	exit 0
elif [ "$1" = "stop" ] ; then
	$CTLSCRIPT stop
	pkill -9 ushare
	exit 0
elif [ "$1" = "restart" ] ; then
	restart
elif [ "$1" = "autostart" ] ; then
	autostart $2
	exit 0
else
	returnresult false "Invalid Operation" "No Operation"
	exit 2
fi
source $BINDIR/ushare.tmpl > /etc/ushare.conf
restart
