#!/bin/bash

DIRNAME=`dirname $0`
OLDVERSIONFILE="/etc/tonidoplug_release"
VERSIONFILE="/etc/tonidofirmware_release"
TMPUPDATEDIR="${DIRNAME}/update"
UPDATELIST="${TMPUPDATEDIR}/updates.list"

TARGETVERSION=0.3.1.1
CURRENTVERSION=
IMAGETYPE=

UPDATED=
trim()
{
	echo ${1}
}

cleanup()
{
	echo "Removing temporary update files.."
	rm -f ${UPDATELIST}
}
getPlugVersion()
{
	if [ -f ${OLDVERSIONFILE} ]; then
		CURRENTVERSION=
		IMAGETYPE=sheeva
	elif [ -f ${VERSIONFILE} ]; then
		CURRENTVERSION=`cat ${VERSIONFILE}  | grep VERSION= | cut -d '=' -f 2`
		IMAGETYPE=`cat ${VERSIONFILE}  | grep TYPE= | cut -d '=' -f 2`
	else
		CURRENTVERSION=
		IMAGETYPE=
	fi
		
}

updateOldPlug()
{
	UPDTFILENAME="updates.tgz"
	UPDTFILEPATH="$DIRNAME/update/$UPDTFILENAME"
	if [[ -f $UPDTFILEPATH ]]; then
		echo -n "TonidoPlug v1: Found image updates. Updating.... "
		tar zxf $UPDTFILEPATH  -C / >& /dev/null
		rm -f $UPDTFILEPATH
		echo "..done."
	else
		echo "TonidoPlug v1: No image updates necessary"
	fi
}

downloadUpdates()
{
	echo "Downloading file http://patch.codelathe.com/firmware/${IMAGETYPE}/update_${1}.tgz from update server"
	wget --directory-prefix=${TMPUPDATEDIR} http://patch.codelathe.com/firmware/${IMAGETYPE}/update_${1}.tgz
	wget --directory-prefix=${TMPUPDATEDIR} http://patch.codelathe.com/firmware/${IMAGETYPE}/update_${1}.md5
	if [ ! -f ${TMPUPDATEDIR}/update_${1}.tgz ]; then
		echo "Unable to download image update version ${1}. Exiting..."
		exit 1
	fi
	if [ ! -f ${TMPUPDATEDIR}/update_${1}.md5 ]; then
		echo "Unable to download checksum for image ${1}. Exiting..."
		exit 1
	fi
	
	FILEMD5=`(cd  ${TMPUPDATEDIR}; md5sum update_${1}.tgz | awk ' { print $1 } ')`
	STOREMD5=`cat ${TMPUPDATEDIR}/update_${1}.md5 | awk ' { print $1 } '`
	
	if [ "${FILEMD5}" != "${STOREMD5}" ]; then
		echo "Checksum mismatch for image ${1}. Exiting..."
		exit 1
	fi
	UPDATED=1
}

installUpdates()
{
	echo -n "Installing update update_${1}.tgz"
	if [[ -f ${TMPUPDATEDIR}/update_${1}.tgz ]]; then
		tar zxf ${TMPUPDATEDIR}/update_${1}.tgz  -C / >& /dev/null
		rm -f ${TMPUPDATEDIR}/update_${1}.tgz
		rm -f ${TMPUPDATEDIR}/update_${1}.md5
		echo "..done."
	else
		echo "Update file ${TMPUPDATEDIR}/update_${1}.tgz File "
	fi
}

processUpgrade()
{
	START=
	CMD="download"	
	if [ ! -z ${1} ]; then
		CMD=${1}
	fi
	while read line
	do 
		VERSION=`trim ${line}`
		if [ ! -z ${START} ]; then
			if [ "${CMD}" = "download" ]; then
				downloadUpdates $VERSION;
			elif [ "${CMD}" = "install" ]; then
				installUpdates $VERSION;
			fi
		fi
		if [ "${VERSION}" = "${CURRENTVERSION}" ]; then
			START=1
		fi
		if [ "${VERSION}" = "${TARGETVERSION}" ]; then
			START=
		fi
		
	done < ${UPDATELIST}
}

getUpdateList()
{
	echo "Getting update list from http://patch.codelathe.com/firmware/${IMAGETYPE}/updates.list"
	wget --directory-prefix=${TMPUPDATEDIR} http://patch.codelathe.com/firmware/${IMAGETYPE}/updates.list
	if [ ! -f ${UPDATELIST} ]; then
		echo "Unable to get image update information. Exiting..."
		exit 1
	fi
}

getPlugVersion

if [ -z ${IMAGETYPE} ]; then
	echo "Invalid image type. Unable to check for updates!"
	exit 1
fi

if [ ${IMAGETYPE} = "sheeva" ]; then
	updateOldPlug
	exit 0
fi

if [ -z ${CURRENTVERSION} ]; then
	echo "Missing image version. Unable to check for updates!"
	exit 1
fi

# update of newer images
mkdir -p ${TMPUPDATEDIR}
rm -f ${TMPUPDATEDIR}/*

getUpdateList
processUpgrade download
processUpgrade install
cleanup

if [ -z ${UPDATED} ]; then
	echo "Image current. No updates necessary"
fi
