#! /bin/bash
# Params: 
# $1 - <connection_mode>
# $2 - <ssid>
# $3 - <password>
# $4 - <encryption type> AES/TKIP

DIRNAME=`dirname $0`
DEBUGLOG="$DIRNAME/wireless.log"
source $DIRNAME/default.properties

WPACONFIG="$DIRNAME/current_wpa.conf"


# Log message
echoLog()
{
	TIMESTAMP=`date +"%b %d %T : "`
	echo "$TIMESTAMP"$@ >> $DEBUGLOG
}

# Write into config file
echoConfig()
{
	echo "$@" >> $WPACONFIG
}

clearConfig()
{
	echo > $WPACONFIG
}

wpa_config_header()
{
	clearConfig
	echoConfig "#$1"
	echoConfig "ctrl_interface=/var/run/wpa_supplicant"	
}

wpa_config_content()
{
	echoConfig "network={"
	echoConfig "	ssid=\"$1\""
	echoConfig "	key_mgmt=$2"
	[[ ! -z "$3" ]] && echoConfig "	proto=$3"
	[[ ! -z "$4" ]] && echoConfig "	pairwise=$4"
    [[ ! -z "$5" ]] && echoConfig "	group=$5"
    [[ ! -z "$6" ]] && echoConfig "	#psk=$6"
    [[ ! -z "$7" ]] && echoConfig "	psk=$7"
    echoConfig "}"
}

passphrase()
{
	eval `wpa_passphrase $1 $2 | sed -e "/#psk/d" | grep psk`
	echo $psk
}

plainconnect()
{
	wpa_config_header "# Plaintext (no encryption) network"
	#wpa_config_content $2 "NONE" "$1"
	wpa_config_content $2 "NONE"
}

cryptconnect()
{
	wpa_config_header "# $1/$4"
	PASSPHRASE=`passphrase $2 $3`
	case "$4" in
		AES)
			wpa_config_content $2 "WPA-PSK" "$1" "CCMP" "CCMP" $3 "$PASSPHRASE"
	        ;;
		TKIP)
			wpa_config_content $2 "WPA-PSK" "$1" "TKIP" "TKIP" $3 "$PASSPHRASE"
	        ;;
		TKIPAES)
			wpa_config_content $2 "WPA-PSK" "$1" "CCMP TKIP" "CCMP TKIP" $3 "$PASSPHRASE"
	        ;;
		*)
			wpa_config_content $2 "NONE" "$1" "NONE" "NONE" $3 "$PASSPHRASE"
	        ;;
	esac
}

echoLog "createWirelessInfo.sh $@"

case "$1" in
  WEP)
  		# Only used to create config. Not used, coz the driver support is not there.
  		cryptconnect $@
        ;;
  WPA)
  		cryptconnect $@
        ;;
  WPA2)
       	cryptconnect $@
        ;;
  PLAIN)
       	plainconnect $@
  		;;
  *)
        echo "Usage: $0 security_mode ssid <key> <encryption_type>" >&2
        exit 1
        ;;
esac

exit 0

