#! /bin/bash

DIRNAME=`dirname $0`
WPACONFIG="$DIRNAME/current_wpa.conf"
MODEFILE="$DIRNAME/current.state"

source $DIRNAME/default.properties

create_new_state_file()
{
	cat > $MODEFILE << EOF
MODE=disabled
ADDRESSTYPE=dhcp
ADDRESS=192.168.1.177
NETMASK=255.255.255.0
GATEWAY=192.168.1.1
APINFONO=1
EOF
}

wpa_client_status()
{
    if [[ ! -S /var/run/wpa_supplicant/$INTERFACE ]]; then
        echo -n "Disconnected"
    else
        eval `wpa_cli status|grep wpa_state`
                case "$wpa_state" in
                        SCANNING)
                        echo -n "Connecting"
                ;;
                        COMPLETED)
                        echo -n "Connected"
                ;;
                        *)
                        echo -n "Unknown"
                ;;
                esac
        fi
}

wpa_ap_status()
{
	STATUS=`/etc/init.d/dnsmasq status | grep \(running\)`	
	if [[ -z $STATUS ]]; then
		echo -n "Not Enabled"
	else
		echo -n "Enabled"
	fi
}

wpa_ap_dhcp_info()
{
	for i in 1 2 3
	do
		APIP=`echo ${APSUBNET[$i]} | cut -d '|' -f 1`
		APDHCPINFO=`echo ${APSUBNET[$i]} | cut -d '|' -f 2`
		STARTIP=`echo $APDHCPINFO | cut -d ',' -f 1` 
		ENDIP=`echo $APDHCPINFO | cut -d ',' -f 2` 
		if [[ ! -z $APIP ]]; then
			echo "<dhcpinfo$i>"$APIP" | {"$STARTIP"-"$ENDIP"}</dhcpinfo$i>"
		fi
	done
	
	if [[ -f $MODEFILE ]]; then
		APINFONO="`cat $MODEFILE | grep APINFONO | sed -e 's/^.*APINFONO=//g'`"
	fi  
	
	if [[ -z $APINFONO ]]; then
		echoLog "Access point dhcp range not found. Using defaults..."
		APINFONO=1
	fi
	echo "<selecteddhcpno>"$APINFONO"</selecteddhcpno>"
	
}

wpa_get_config()
{
	echo "<info><config>"

	if [[ -f /sys/class/net/$INTERFACE/address ]]; then
		MAC=`cat /sys/class/net/$INTERFACE/address`
		echo "<mac>$MAC</mac>"
	fi

	if [[ ! -f $MODEFILE ]]; then
		create_new_state_file
        fi

	
	if [[ ! -f $WPACONFIG ]]; then
		echo "</config></info>"
		return
	fi
	
	eval `cat $MODEFILE | grep MODE`
	echo "<mode>$MODE</mode>"

	SSID=`cat $WPACONFIG | grep ssid | sed -e 's/^.*ssid=\"//g' -e 's/\"$//g'`
	echo "<ssid>$SSID</ssid>"
	
	
	PROTO=`cat $WPACONFIG | grep proto | sed -e 's/^.*proto=//g'`
	echo "<proto>$PROTO</proto>"		
	
	ENC=`cat $WPACONFIG | grep pairwise | sed -e 's/^.*pairwise=//g'`
	if [[ "$ENC" = "CCMP TKIP" ]]; then
		echo "<encryption>TKIPAES</encryption>"		
	elif [[ "$ENC" = "CCMP" ]]; then
		echo "<encryption>AES</encryption>"
	else
		echo "<encryption>$ENC</encryption>"
	fi
	
	KEY=`cat $WPACONFIG | grep \#psk | sed -e 's/^.*#psk=//g'`
	if [[ ! -z $KEY ]]; then
		echo "<key>$KEY</key>"	
	fi
		
	echo -n "<status>"
	if [[ $MODE = "client" ]]; then		
		wpa_client_status
	elif [[ $MODE = "accesspoint" ]]; then		
		wpa_ap_status
	else
		echo "N.A."
	fi
	echo "</status>"
	
	wpa_ap_dhcp_info
	
	echo "</config></info>"
}

get_ip_config()
{
	echo "<ipconfig><information>"
	IPINFO=`ifconfig $INTERFACE 2>/dev/null | grep inet `

	if [ ! -z "$IPINFO" ] 
	then
		eval `cat $MODEFILE | grep ADDRESSTYPE`
		echo "<type>$ADDRESSTYPE</type>"
		echo "<ip>"
		ifconfig $INTERFACE | grep inet | awk ' { print $2 } ' | cut -d: -f 2
		echo "</ip>"
		echo "<mask>"
		ifconfig $INTERFACE | grep inet | awk ' { print $4 } ' | cut -d: -f 2
		echo "</mask>"
		echo "<gw>"
		route -n | grep $INTERFACE |grep "^0.0.0.0" | awk ' { print $2 } '
		echo "</gw>"
		i=0
		for ns in `cat /etc/resolv.conf  | grep nameserver | awk ' { print $2 } '`
		do
			i=$(( $i+1 ))
			echo "<ns"$i">"
			echo $ns
			echo "</ns"$i">"
		done
		
	fi
	echo "</information></ipconfig>"
}

get_ap_connected_clients()
{
	echo "<clients>"
	for f in `uaputl sta_list | grep MAC | sed -e 's/MAC Address: //g'`
	do
		echo "<client>"
		echo "<mac>$f</mac>"
		NAME=`cat /var/lib/misc/dnsmasq.leases 2> /dev/null| grep $f | cut -d ' ' -f 4`	
		IP=`cat /var/lib/misc/dnsmasq.leases 2> /dev/null| grep $f | cut -d ' ' -f 3`	
		echo "<name>${NAME}</name>"
		echo "<ip>${IP}</ip>"
		echo "</client>"
	done
	echo "</clients>"
		
}
case "$1" in
  	config)
  		wpa_get_config
  		;;
	ip)
  		get_ip_config
  		;;
	apclients)
		get_ap_connected_clients
		;;
  *)
        echo "Usage: $0 config|ip" >&2
        exit 1
        ;;
esac

exit 0

